package de.fitko.fitconnect.core.crypto;

import de.fitko.fitconnect.api.exceptions.InitializationException;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

public class HashUtil {

    private static final String DEFAULT_ALGORITHM = "SHA-512"; // Currently, only SHA-512 is supported.

    private final MessageDigest messageDigest;

    public HashUtil() {
        try {
            this.messageDigest = MessageDigest.getInstance(DEFAULT_ALGORITHM);
        } catch (final NoSuchAlgorithmException e) {
            throw new InitializationException(e.getMessage(), e);
        }
    }

    public HashUtil(final MessageDigest messageDigest) {
        this.messageDigest = messageDigest;
    }

    public byte[] createHash(final byte[] data) {
        this.messageDigest.reset();
        return this.messageDigest.digest(data);
    }

    public boolean verify(final byte[] originalHash, final byte[] data) {
        final byte[] newHash = createHash(data);
        return compareHashes(originalHash, newHash);
    }

    public String createHashFromFile(final File file) throws IOException {
        try {
            final byte[] rawData = Files.readAllBytes(Paths.get(file.getPath()));
            return new String(rawData);
        } catch (final IOException e) {
            throw e;
        }
    }

    private boolean compareHashes(final byte[] originalHash, final byte[] comparisonHash) {
        int diff = originalHash.length ^ comparisonHash.length;
        for (int i = 0; i < originalHash.length && i < comparisonHash.length; i++) {
            diff |= originalHash[i] ^ comparisonHash[i];
        }
        return diff == 0;
    }

}
